/*
 
 eos.cpp
 
 Copyright (c) Mohammad Nopoush and Michael Strickland
 
 GNU General Public License (GPLv3)
 See detailed text in license directory
 
 */

#include <fstream>
#include <iostream>
#include <sstream>
#include <cmath>
#include <gsl/gsl_spline.h>

using namespace std;

#include "aHydro1p1b.h"
#include "rfuncs.h"
#include "eos.h"

int nTfnc;
double *TfncX;
double *TfncY;

int nEOS;
double *X;
double *E;
double *DE;
double *P;
double *DP;

gsl_interp_accel *accTfnc;
gsl_spline *splineTfnc;

gsl_interp_accel *accE;
gsl_spline *splineE;

gsl_interp_accel *accDE;
gsl_spline *splineDE;

gsl_interp_accel *accP;
gsl_spline *splineP;

gsl_interp_accel *accDP;
gsl_spline *splineDP;

void readTfnc() {
    ifstream inputFile;
    char fname[255];
    sprintf(fname,"%s/TfuncData.dat",EOSDIR);
    string line;
    inputFile.open(fname);
    getline(inputFile, line); // read the header and discard since we don't need it
    getline(inputFile, line); // read number lines
    istringstream ss(line);
    ss >> nTfnc;
    TfncX = new double[nTfnc];
    TfncY = new double[nTfnc];
    int i = 0;
    while (getline(inputFile, line))
    {
        istringstream ss(line);
        ss >> TfncX[i] >> TfncY[i];
        i++;
    }
    inputFile.close();
}

void readEOS() {
    ifstream inputFile;
    char fname[255];
    sprintf(fname,"%s/eosData.dat",EOSDIR);
    string line;
    inputFile.open(fname);
    getline(inputFile, line); // read the header and discard since we don't need it
    getline(inputFile, line); // read number lines
    istringstream ss(line);
    ss >> nEOS;
    X = new double[nEOS];
    E = new double[nEOS];
    DE = new double[nEOS];
    P = new double[nEOS];
    DP = new double[nEOS];
    int i = 0;
    while (getline(inputFile, line))
    {
        istringstream ss(line);
        ss >> X[i] >> E[i] >> DE[i] >> P[i] >> DP[i];
        i++;
    }
    inputFile.close();
}

void initializeTfncInt() {
    accTfnc = gsl_interp_accel_alloc();
    splineTfnc = gsl_spline_alloc(gsl_interp_cspline, nTfnc);
    gsl_spline_init(splineTfnc, TfncX, TfncY, nTfnc);
}

void initializeEOSInts() {
    accE = gsl_interp_accel_alloc();
    splineE = gsl_spline_alloc(gsl_interp_cspline, nEOS);
    gsl_spline_init(splineE, X, E, nEOS);
    
    accDE = gsl_interp_accel_alloc();
    splineDE = gsl_spline_alloc(gsl_interp_cspline, nEOS);
    gsl_spline_init(splineDE, X, DE, nEOS);
    
    accP = gsl_interp_accel_alloc();
    splineP = gsl_spline_alloc(gsl_interp_cspline, nEOS);
    gsl_spline_init(splineP, X, P, nEOS);

    accDP = gsl_interp_accel_alloc();
    splineDP = gsl_spline_alloc(gsl_interp_cspline, nEOS);
    gsl_spline_init(splineDP, X, DP, nEOS);
}

void initializeEoS() {
    cout << "==> Initializing EoS" << endl;
    readTfnc(); // loads TfncX, TfncY, and nTfnc
    initializeTfncInt(); // load the interpolating function
    readEOS(); // loads X, E, DE, P, DP, and nEOS
    initializeEOSInts();
}

void finalizeEoS() {
    gsl_spline_free(splineTfnc);
    gsl_interp_accel_free(accTfnc);
    gsl_spline_free(splineE);
    gsl_interp_accel_free(accE);
    gsl_spline_free(splineDE);
    gsl_interp_accel_free(accDE);
    gsl_spline_free(splineP);
    gsl_interp_accel_free(accP);
    gsl_spline_free(splineDP);
    gsl_interp_accel_free(accDP);
}

double Eiso(double lambda) {
    if (EOS==1) return pow(10.,gsl_spline_eval(splineE, log10(lambda), accE));
    return EOSFACTOR*pow(lambda,4.);
}

double dEiso(double lambda) {
    if (EOS==1) return pow(10.,gsl_spline_eval(splineDE, log10(lambda), accDE));
    return 4*EOSFACTOR*pow(lambda,3.);
}

double Piso(double lambda) {
    if (EOS==1) return pow(10.,gsl_spline_eval(splineP, log10(lambda), accP));
    return EOSFACTOR*pow(lambda,4.)/3.;
}

double dPiso(double lambda) {
    if (EOS==1) return pow(10.,gsl_spline_eval(splineDP, log10(lambda), accE));
    return 4*EOSFACTOR*pow(lambda,3.)/3.;
}

double Teff(double ed) {
    if (EOS==1) return pow(10.,gsl_spline_eval(splineTfnc, log10(ed), accTfnc));
    else return pow(ed/EOSFACTOR,0.25);
}

double Teff(double ax, double az, double lambda) {
    if (EOS==1) return Teff(R(ax,az)*Eiso(lambda));
    return pow(R(ax,az),0.25)*lambda;
}
