/*
 
 equations.cpp
 
 Copyright (c) Mohammad Nopoush and Michael Strickland
 
 GNU General Public License (GPLv3)
 See detailed text in license directory
 
 */

#include <fstream>
#include <iostream>
#include <cmath>

using namespace std;

#include "aHydro1p1b.h"
#include "rfuncs.h"
#include "eos.h"

#define Power(a,b) pow((double)a,(double)b)

#define UNWRAPVARS struct stateVars *myVars = (struct stateVars *) myStateVars; \
double t = myVars->t; \
double r = myVars->r; \
double lambda = myVars->lambda; \
double alphax = myVars->alphax; \
double alphaz = myVars->alphaz; \
double thetap = myVars->thetap; \
double drlambda = myVars->drlambda; \
double dralphax = myVars->dralphax; \
double dralphaz = myVars->dralphaz; \
double drthetap = myVars->drthetap; \
struct stateFncs *myFncs = (struct stateFncs *) myStateFncs; \
double T = myFncs->T; \
double R = myFncs->R; \
double d1R = myFncs->d1R; \
double d2R = myFncs->d2R; \
double HTx = myFncs->HTx; \
double d1HTx = myFncs->d1HTx; \
double d2HTx = myFncs->d2HTx; \
double HTy = myFncs->HTy; \
double HL = myFncs->HL; \
double ay = myFncs->ay; \
double d1ay = myFncs->d1ay; \
double d2ay = myFncs->d2ay; \
double Eiso = myFncs->Eiso; \
double dEiso = myFncs->dEiso; \
double Piso = myFncs->Piso; \
double dPiso = myFncs->dPiso; \
double sinhThetap = myFncs->sinhThetap; \
double coshThetap = myFncs->coshThetap; \
double teq = 5*ETABAR*(1+Eiso/Piso)/T/4;

void loadStateFncs(stateVars * myStateVars, stateFncs * myStateFncs) {
    
    struct stateVars *myVars = (struct stateVars *) myStateVars;
    double t = myVars->t;
    double r = myVars->r;
    double lambda = myVars->lambda;
    double alphax = myVars->alphax;
    double alphaz = myVars->alphaz;
    double thetap = myVars->thetap;
    double drlambda = myVars->drlambda;
    double dralphax = myVars->dralphax;
    double dralphaz = myVars->dralphaz;
    double drthetap = myVars->drthetap;

    myStateFncs->T = Teff(alphax,alphaz,lambda);
    myStateFncs->R = R(alphax,alphaz);
    myStateFncs->d1R = d1R(alphax,alphaz);
    myStateFncs->d2R = d2R(alphax,alphaz);
    myStateFncs->HTx = HTx(alphax,alphaz);
    myStateFncs->d1HTx = d1HTx(alphax,alphaz);
    myStateFncs->d2HTx = d2HTx(alphax,alphaz);
    myStateFncs->HTy = HTy(alphax,alphaz);
    myStateFncs->HL = HL(alphax,alphaz);
    myStateFncs->ay = ay(alphax,alphaz);
    myStateFncs->d1ay = d1ay(alphax,alphaz);
    myStateFncs->d2ay = d2ay(alphax,alphaz);
    myStateFncs->Eiso = Eiso(lambda);
    myStateFncs->dEiso = dEiso(lambda);
    myStateFncs->Piso = Piso(lambda);
    myStateFncs->dPiso = dPiso(lambda);
    myStateFncs->sinhThetap = sinh(thetap);
    myStateFncs->coshThetap = cosh(thetap);
    
    return;
}

double lambdaRHS(stateVars * myStateVars, stateFncs * myStateFncs) {
    UNWRAPVARS
    return -((4*((3*HTy*Piso + d1R*dralphax*Eiso*r + d2R*dralphaz*Eiso*r + Eiso*R + dEiso*drlambda*r*R)*sinhThetap*t + coshThetap*r*(3*HL*Piso + Eiso*R + 3*drthetap*HTx*Piso*t + drthetap*Eiso*R*t)))/(r*t) + (3*HTx*Piso + Eiso*R)*(-4*coshThetap*drthetap - (4*dralphax*sinhThetap)/alphax + (2*dralphaz*sinhThetap)/alphaz + (2*(d1ay*dralphax + d2ay*dralphaz)*sinhThetap)/ay + (2*sinhThetap)/r + (2*coshThetap)/t - (3*(-1 + Power(alphax,2))*Power(T,5))/(Power(alphax,3)*alphaz*ay*Power(lambda,5)*teq)) - ((alphaz*d2ay*(3*HTx*Piso + Eiso*R) + ay*(2*alphaz*d2R*Eiso + 3*HTx*Piso + Eiso*R))* (Power(ay,2)*r*t*Power(T,5) + Power(alphax,2)*(2 - 3*Power(ay,2))*r*t*Power(T,5) + 2*Power(alphax,3)*Power(ay,2)*Power(lambda,5)*(-(alphaz*(d1ay*dralphax + d2ay*dralphaz)*r*sinhThetap*t) + ay*(alphaz*coshThetap*r - alphaz*sinhThetap*t + dralphaz*r*sinhThetap*t))*teq))/(Power(alphax,3)*alphaz*Power(ay,3)*(ay - alphaz*d2ay)*Power(lambda,5)*r*t*teq) + ((-((ay - alphaz*d2ay)*(3*HTx*Piso + Eiso*R)) + alphax*(ay*d1R*Eiso - alphaz*d1R*d2ay*Eiso + alphaz*d1ay*d2R*Eiso + 3*d1ay*HTx*Piso + d1ay*Eiso*R))* (-3*alphaz*Power(ay,3)*(ay - alphaz*d2ay)*((4*sinhThetap*(-3*Power(coshThetap,2)*dEiso* (Piso*(-HTy + d1HTx*dralphax*r + d2HTx*dralphaz*r) + HTx*(Piso + dPiso*drlambda*r))*R*t + 3*dPiso*HTx*(3*HTy*Piso + d1R*dralphax*Eiso*r + d2R*dralphaz*Eiso*r + Eiso*R + dEiso*drlambda*r*R)*Power(sinhThetap,2)*t + coshThetap*r*sinhThetap*(3*HL*Piso*(3*dPiso*HTx + dEiso*R) - dEiso*R*(drthetap*Eiso*R*t + 3*HTx*(Piso + drthetap*Piso*t)) + 3*dPiso*HTx*(3*drthetap*HTx*Piso*t + Eiso*(R + drthetap*R*t)))))/(3.*r*t) - ((3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2))* (-4*coshThetap*drthetap - (4*dralphax*sinhThetap)/alphax + (2*dralphaz*sinhThetap)/alphaz + (2*(d1ay*dralphax + d2ay*dralphaz)*sinhThetap)/ay + (2*sinhThetap)/r + (2*coshThetap)/t - (3*(-1 + Power(alphax,2))*Power(T,5))/(Power(alphax,3)*alphaz*ay*Power(lambda,5)*teq)))/3.) - ((alphaz*d2ay*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2)) + ay*(Power(coshThetap,2)*dEiso*R*(3*HTx*Piso + Eiso*R) - 3*(2*alphaz*d2R*dPiso*Eiso*HTx + 3*dPiso*Power(HTx,2)*Piso + dPiso*Eiso*HTx*R - 2*alphaz*d2HTx*dEiso*Piso*R)*Power(sinhThetap,2)))* (Power(ay,2)*r*t*Power(T,5) + Power(alphax,2)*(2 - 3*Power(ay,2))*r*t*Power(T,5) + 2*Power(alphax,3)*Power(ay,2)*Power(lambda,5)*(-(alphaz*(d1ay*dralphax + d2ay*dralphaz)*r*sinhThetap*t) + ay*(alphaz*coshThetap*r - alphaz*sinhThetap*t + dralphaz*r*sinhThetap*t))*teq))/(Power(alphax,3)*Power(lambda,5)*r*t*teq)))/ (alphaz*Power(ay,3)*(ay - alphaz*d2ay)*(alphaz*d2ay*(3*HTx*Piso + Eiso*R)*(-(Power(coshThetap,2)*dEiso*R) + 3*dPiso*HTx*Power(sinhThetap,2)) + ay*(Power(coshThetap,2)*dEiso*R*(3*HTx*Piso + Eiso*R) - 3*(-(alphax*d1R*dPiso*Eiso*HTx) + 3*dPiso*Power(HTx,2)*Piso + dPiso*Eiso*HTx*R + alphax*d1HTx*dEiso*Piso*R)*Power(sinhThetap,2)) + alphax* (-(Power(coshThetap,2)*d1ay*dEiso*R*(3*HTx*Piso + Eiso*R)) + 3*(d1ay*dPiso*HTx*(3*HTx*Piso + Eiso*R) + alphaz*(-(d1R*d2ay*dPiso*Eiso*HTx) + d1ay*d2R*dPiso*Eiso*HTx + d1HTx*d2ay*dEiso*Piso*R - d1ay*d2HTx*dEiso*Piso*R))* Power(sinhThetap,2)))))/(4.*coshThetap*dEiso*R);
}

double alphaxRHS(stateVars * myStateVars, stateFncs * myStateFncs) {
    UNWRAPVARS
    return -(alphax*(-3*alphaz*Power(ay,3)*(ay - alphaz*d2ay)*((4*sinhThetap*(-3*Power(coshThetap,2)*dEiso* (Piso*(-HTy + d1HTx*dralphax*r + d2HTx*dralphaz*r) + HTx*(Piso + dPiso*drlambda*r))*R*t + 3*dPiso*HTx*(3*HTy*Piso + d1R*dralphax*Eiso*r + d2R*dralphaz*Eiso*r + Eiso*R + dEiso*drlambda*r*R)*Power(sinhThetap,2)*t + coshThetap*r*sinhThetap*(3*HL*Piso*(3*dPiso*HTx + dEiso*R) - dEiso*R*(drthetap*Eiso*R*t + 3*HTx*(Piso + drthetap*Piso*t)) + 3*dPiso*HTx*(3*drthetap*HTx*Piso*t + Eiso*(R + drthetap*R*t)))))/(3.*r*t) - ((3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2))* (-4*coshThetap*drthetap - (4*dralphax*sinhThetap)/alphax + (2*dralphaz*sinhThetap)/alphaz + (2*(d1ay*dralphax + d2ay*dralphaz)*sinhThetap)/ay + (2*sinhThetap)/r + (2*coshThetap)/t - (3*(-1 + Power(alphax,2))*Power(T,5))/(Power(alphax,3)*alphaz*ay*Power(lambda,5)*teq)))/3.) - ((alphaz*d2ay*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2)) + ay*(Power(coshThetap,2)*dEiso*R*(3*HTx*Piso + Eiso*R) - 3* (2*alphaz*d2R*dPiso*Eiso*HTx + 3*dPiso*Power(HTx,2)*Piso + dPiso*Eiso*HTx*R - 2*alphaz*d2HTx*dEiso*Piso*R)*Power(sinhThetap,2)))* (Power(ay,2)*r*t*Power(T,5) + Power(alphax,2)*(2 - 3*Power(ay,2))*r*t*Power(T,5) + 2*Power(alphax,3)*Power(ay,2)*Power(lambda,5)*(-(alphaz*(d1ay*dralphax + d2ay*dralphaz)*r*sinhThetap*t) + ay*(alphaz*coshThetap*r - alphaz*sinhThetap*t + dralphaz*r*sinhThetap*t))*teq))/(Power(alphax,3)*Power(lambda,5)*r*t*teq)))/ (4.*alphaz*Power(ay,3)*coshThetap*(alphaz*d2ay*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2)) + ay*(-(Power(coshThetap,2)*dEiso*R*(3*HTx*Piso + Eiso*R)) + 3*(-(alphax*d1R*dPiso*Eiso*HTx) + 3*dPiso*Power(HTx,2)*Piso + dPiso*Eiso*HTx*R + alphax*d1HTx*dEiso*Piso*R)*Power(sinhThetap,2)) + alphax* (Power(coshThetap,2)*d1ay*dEiso*R*(3*HTx*Piso + Eiso*R) + 3*(-(d1ay*dPiso*HTx*(3*HTx*Piso + Eiso*R)) + alphaz*(d1R*d2ay*dPiso*Eiso*HTx - d1ay*d2R*dPiso*Eiso*HTx - d1HTx*d2ay*dEiso*Piso*R + d1ay*d2HTx*dEiso*Piso*R))*Power(sinhThetap,2))));
}

double alphazRHS(stateVars * myStateVars, stateFncs * myStateFncs) {
    UNWRAPVARS
    return (Power(ay,3)*r*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2))*t*Power(T,5) + Power(alphax,2)*ay*(-2 + 3*Power(ay,2))*r*(3*HTx*Piso + Eiso*R)*(-(Power(coshThetap,2)*dEiso*R) + 3*dPiso*HTx*Power(sinhThetap,2))*t*Power(T,5) + alphax*Power(ay,2)*r*(-2*Power(coshThetap,2)*d1ay*dEiso*R*(3*HTx*Piso + Eiso*R) + 3*(2*d1ay*dPiso*HTx*(3*HTx*Piso + Eiso*R) + ay*(d1R*dPiso*Eiso*HTx - d1HTx*dEiso*Piso*R))*Power(sinhThetap,2))*t*Power(T,5) - 2*Power(alphax,4)*Power(ay,3)*Power(lambda,5)*(dralphaz*r*sinhThetap* (Power(coshThetap,2)*d1ay*dEiso*R*(3*HTx*Piso + Eiso*R) - 3*(d1ay*dPiso*HTx*(3*HTx*Piso + Eiso*R) + ay*(d1R*dPiso*Eiso*HTx - d1HTx*dEiso*Piso*R))* Power(sinhThetap,2))*t - alphaz*(-3*Power(coshThetap,2)*d1ay*dEiso*(Piso*(-HTy + d1HTx*dralphax*r + d2HTx*dralphaz*r) + HTx*(Piso + dPiso*drlambda*r))*R* sinhThetap*t + 3*(ay*(-(d1R*dPiso*Eiso*HTx) + d1HTx*dEiso*Piso*R) + d2ay*dralphaz*r*(-(d1R*dPiso*Eiso*HTx) + d1HTx*dEiso*Piso*R) + d1ay*(d1HTx*dEiso*dralphax*Piso*r*R + dPiso*HTx*(3*HTy*Piso + d2R*dralphaz*Eiso*r + Eiso*R + dEiso*drlambda*r*R)))*Power(sinhThetap,3)*t + Power(coshThetap,3)*d1ay*dEiso*r*R*(3*HTx*Piso + Eiso*R)*(-1 + drthetap*t) + coshThetap*r*Power(sinhThetap,2)*(3*ay*(d1R*dPiso*Eiso*HTx - d1HTx*dEiso*Piso*R) + d1ay*(9*dPiso*Power(HTx,2)*Piso + 3*HL*Piso*(3*dPiso*HTx + dEiso*R) - dEiso*drthetap*Eiso*Power(R,2)*t - 3*HTx*R*(-2*dPiso*Eiso + dEiso*Piso + dEiso*drthetap*Piso*t)))))*teq + Power(alphax,3)*(6*ay*r*(d1R*dPiso*Eiso*HTx - d1HTx*dEiso*Piso*R)*Power(sinhThetap,2)*t*Power(T,5) + 3*Power(ay,2)*d1ay*r*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2))*t*Power(T,5) + d1ay*r*(3*HTx*Piso + Eiso*R)*(-(Power(coshThetap,2)*dEiso*R) + 3*dPiso*HTx*Power(sinhThetap,2))*t*Power(T,5) + 2*Power(ay,4)*Power(lambda,5)*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2))* (alphaz*coshThetap*r - alphaz*sinhThetap*t + dralphaz*r*sinhThetap*t)*teq + Power(ay,3)*r*sinhThetap*t*(9*(-(d1R*dPiso*Eiso*HTx) + d1HTx*dEiso*Piso*R)*sinhThetap*Power(T,5) + 2*alphaz*d2ay*dralphaz*Power(lambda,5)*(3*HTx*Piso + Eiso*R)*(-(Power(coshThetap,2)*dEiso*R) + 3*dPiso*HTx*Power(sinhThetap,2))*teq)))/ (2.*Power(alphax,3)*Power(ay,3)*coshThetap*Power(lambda,5)*r*(alphaz*d2ay*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2)) + ay*(-(Power(coshThetap,2)*dEiso*R*(3*HTx*Piso + Eiso*R)) + 3*(-(alphax*d1R*dPiso*Eiso*HTx) + 3*dPiso*Power(HTx,2)*Piso + dPiso*Eiso*HTx*R + alphax*d1HTx*dEiso*Piso*R)*Power(sinhThetap,2)) + alphax* (Power(coshThetap,2)*d1ay*dEiso*R*(3*HTx*Piso + Eiso*R) + 3*(-(d1ay*dPiso*HTx*(3*HTx*Piso + Eiso*R)) + alphaz*(d1R*d2ay*dPiso*Eiso*HTx - d1ay*d2R*dPiso*Eiso*HTx - d1HTx*d2ay*dEiso*Piso*R + d1ay*d2HTx*dEiso*Piso*R))*Power(sinhThetap,2)))*t*teq);
}

double thetapRHS(stateVars * myStateVars, stateFncs * myStateFncs) {
    UNWRAPVARS
    return ((2*(Power(ay,2)*(ay - 2*alphaz*d2ay)*r*t*Power(T,5) + 2*Power(alphax,3)*alphaz*Power(ay,3)*Power(lambda,5)* ((-(ay*coshThetap*drthetap*r) + ay*sinhThetap + d1ay*dralphax*r*sinhThetap)*t + alphaz*coshThetap*d2ay*r*(-1 + drthetap*t))*teq - Power(alphax,2)*r*t*(ay*Power(T,5) + alphaz*d2ay*Power(T,5) - 3*alphaz*Power(ay,2)*d2ay*Power(T,5) + 2*alphaz*Power(ay,4)*dralphax*Power(lambda,5)*sinhThetap*teq - 2*Power(alphaz,2)*Power(ay,3)*d2ay*dralphax*Power(lambda,5)*sinhThetap*teq)))/ (Power(lambda,5)*r*t*teq) - (Power(alphax,3)*(-ay + alphax*d1ay + alphaz*d2ay)* (-3*alphaz*Power(ay,3)*(ay - alphaz*d2ay)*((4*sinhThetap*(-3*Power(coshThetap,2)*dEiso* (Piso*(-HTy + d1HTx*dralphax*r + d2HTx*dralphaz*r) + HTx*(Piso + dPiso*drlambda*r))*R*t + 3*dPiso*HTx*(3*HTy*Piso + d1R*dralphax*Eiso*r + d2R*dralphaz*Eiso*r + Eiso*R + dEiso*drlambda*r*R)*Power(sinhThetap,2)*t + coshThetap*r*sinhThetap*(3*HL*Piso*(3*dPiso*HTx + dEiso*R) - dEiso*R*(drthetap*Eiso*R*t + 3*HTx*(Piso + drthetap*Piso*t)) + 3*dPiso*HTx*(3*drthetap*HTx*Piso*t + Eiso*(R + drthetap*R*t)))))/(3.*r*t) - ((3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2))* (-4*coshThetap*drthetap - (4*dralphax*sinhThetap)/alphax + (2*dralphaz*sinhThetap)/alphaz + (2*(d1ay*dralphax + d2ay*dralphaz)*sinhThetap)/ay + (2*sinhThetap)/r + (2*coshThetap)/t - (3*(-1 + Power(alphax,2))*Power(T,5))/(Power(alphax,3)*alphaz*ay*Power(lambda,5)*teq)))/3.) - ((alphaz*d2ay*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2)) + ay*(Power(coshThetap,2)*dEiso*R*(3*HTx*Piso + Eiso*R) - 3*(2*alphaz*d2R*dPiso*Eiso*HTx + 3*dPiso*Power(HTx,2)*Piso + dPiso*Eiso*HTx*R - 2*alphaz*d2HTx*dEiso*Piso*R)*Power(sinhThetap,2)))* (Power(ay,2)*r*t*Power(T,5) + Power(alphax,2)*(2 - 3*Power(ay,2))*r*t*Power(T,5) + 2*Power(alphax,3)*Power(ay,2)*Power(lambda,5)*(-(alphaz*(d1ay*dralphax + d2ay*dralphaz)*r*sinhThetap*t) + ay*(alphaz*coshThetap*r - alphaz*sinhThetap*t + dralphaz*r*sinhThetap*t))*teq))/(Power(alphax,3)*Power(lambda,5)*r*t*teq)))/ (alphaz*d2ay*(3*HTx*Piso + Eiso*R)*(Power(coshThetap,2)*dEiso*R - 3*dPiso*HTx*Power(sinhThetap,2)) + ay*(-(Power(coshThetap,2)*dEiso*R*(3*HTx*Piso + Eiso*R)) + 3*(-(alphax*d1R*dPiso*Eiso*HTx) + 3*dPiso*Power(HTx,2)*Piso + dPiso*Eiso*HTx*R + alphax*d1HTx*dEiso*Piso*R)*Power(sinhThetap,2)) + alphax* (Power(coshThetap,2)*d1ay*dEiso*R*(3*HTx*Piso + Eiso*R) + 3*(-(d1ay*dPiso*HTx*(3*HTx*Piso + Eiso*R)) + alphaz*(d1R*d2ay*dPiso*Eiso*HTx - d1ay*d2R*dPiso*Eiso*HTx - d1HTx*d2ay*dEiso*Piso*R + d1ay*d2HTx*dEiso*Piso*R))*Power(sinhThetap,2))))/ (4.*Power(alphax,3)*alphaz*Power(ay,3)*(ay - alphaz*d2ay)*sinhThetap);
}
