/*
 
 outputroutines.cpp
 
 Copyright (c) Mohammad Nopoush and Michael Strickland
 
 GNU General Public License (GPLv3)
 See detailed text in license directory
 
 */

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <gsl/gsl_sf_zeta.h>

using namespace std;

#include "aHydro1p1b.h"
#include "outputroutines.h"
#include "rfuncs.h"
#include "eos.h"

double ctrVal(double *array) {
    int h = NUM/2;
    return (array[h]+array[h-1])/2.;
}

void outputMeasurements(const double time) {
    
    int h = NUM/2;
    
    double l = ctrVal(lambda);
    double ax = ctrVal(alphax);
    double az = ctrVal(alphaz);
    double tp = ctrVal(thetap);
    double teff = Teff(ax,az,l);
    
    // output to screen
    cout.precision(12);
    cout.width(dwidth); cout << time;
    cout.width(dwidth); cout << l*HBARC;
    cout.width(dwidth); cout << ax;
    cout.width(dwidth); cout << az;
    cout.width(dwidth); cout << teff*HBARC;
    cout << endl;
    
    // output to files
    monitor_out << time << "\t";
    monitor_out << l*HBARC << "\t";
    monitor_out << ax << "\t";
    monitor_out << az << "\t";
    monitor_out << teff*HBARC << "\t";
    monitor_out << endl;
    
}

void outputSnapshot(double *data, double time, double fac, string label) {
    double r;
    fstream out;
    char fname[255];
    
    sprintf(fname,"output/snapshot/%s_%.3f.dat",label.c_str(),time);
    out.open(fname, ios::out);
    for (int s=0; s<NUM; s++) {
        r = (s - ((double)NUM-1)/2.)*A; // r in fm
        out << r << "\t" << data[s]*fac << endl;
    }
    out.close();
    return;
}

void outputTeff(double time) {
    double r;
    fstream out;
    char fname[255];
    
    sprintf(fname,"output/snapshot/teff_%.3f.dat",time);
    out.open(fname, ios::out);
    for (int s=0; s<NUM; s++) {
        r = (s - ((double)NUM-1)/2.)*A; // r in fm
        out << r << "\t" << Teff(alphax[s],alphaz[s],lambda[s])*HBARC << endl;
    }
    out.close();
    return;
}

void print_line() {
    for (int i=0;i<5*dwidth;i++) cout << "-"; cout << endl;
    return;
}

