/*
 
 paramreader.cpp
 
 Copyright (c) Mohammad Nopoush and Michael Strickland
 
 GNU General Public License (GPLv3)
 See detailed text in license directory
 
 */

#include <fstream>
#include <iostream>
#include <string>
#include <cstring>
#include <cmath>
#include <stdlib.h>

#include "aHydro1p1b.h"

using namespace std;

// this workhorse examines a key to see if it corresponds to a var we are setting
// and then attempts to set the var corresponding to key by converting value to the
// appropriate type.  lots of hardcoding here
void setParameter(const char *key, const char *value) {
    // integer params
    if (strcmp(key,"NUM")==0) NUM=atoi(value);
    if (strcmp(key,"STEPS")==0) STEPS=atoi(value);
    if (strcmp(key,"UPDATE")==0) UPDATE=atoi(value);
    if (strcmp(key,"SNAPUPDATE")==0) SNAPUPDATE=atoi(value);
    if (strcmp(key,"INITCONDTYPE")==0) INITCONDTYPE=atoi(value);
    if (strcmp(key,"STEPPER")==0) STEPPER=atoi(value);
    if (strcmp(key,"IDEAL")==0) IDEAL=atoi(value);
    if (strcmp(key,"USELAX")==0) USELAX=atoi(value);
    if (strcmp(key,"EOS")==0) EOS=atoi(value);
    if (strcmp(key,"NA")==0) NA=atoi(value);
    if (strcmp(key,"NB")==0) NB=atoi(value);
    if (strcmp(key,"COMPUTEFREEZEOUT")==0) COMPUTEFREEZEOUT=atoi(value);
    if (strcmp(key,"ISOFREEZE")==0) ISOFREEZE=atoi(value);
    // double/float params
    if (strcmp(key,"A")==0) A=atof(value);
    if (strcmp(key,"EPS")==0) EPS=atof(value);
    if (strcmp(key,"TAU0")==0) TAU0=atof(value);
    if (strcmp(key,"L0")==0) L0=atof(value)/HBARC;
    if (strcmp(key,"ALPHAZ0")==0) ALPHAZ0=atof(value);
    if (strcmp(key,"ALPHAX0")==0) ALPHAX0=atof(value);
    if (strcmp(key,"FPIETABAR")==0) ETABAR=atof(value)/(4*M_PI);
    if (strcmp(key,"LAXFRAC")==0) LAXFRAC=atof(value);
    if (strcmp(key,"SIGMANN")==0) SIGMANN=atof(value);
    if (strcmp(key,"Q")==0) Q=atof(value);
    if (strcmp(key,"TFO")==0) TFO=atof(value)/HBARC;
    // string params
    if (strcmp(key,"EOSDIR")==0) strcpy(EOSDIR,value);
    if (strcmp(key,"PARTICLEFILE")==0) strcpy(PARTICLEFILE,value);
    return;
}

//
// This routine assumes that parameters are in text file with
// each parameter on a new line in the format
//
// PARAMKEY	PARAMVALUE
//
// The PARAMKEY must begin the line and only tabs and spaces
// can appear between the PARAMKEY and PARAMVALUE.
//
// Lines which begin with 'commentmarker' defined below are ignored
//
void readParametersFromFile(string filename, int echo) {
    
    string commentmarker = "//";
    char space = ' ';
    char tab = '\t';
    
    int maxline = 128; // maximum line length used in the buffer for reading
    char buffer[maxline];
    ifstream paramFile(filename.c_str());
    
    while(!paramFile.eof()) {
        paramFile.getline(buffer,maxline,'\n');
        string line = buffer; int length = strlen(buffer);
        if (line.substr(0,commentmarker.length())!=commentmarker && line.length()>0) {
            char key[64]="",value[64]="";
            int founddelim=0;
            for (int i=0;i<length;i++) {
                if (buffer[i]==space || buffer[i]==tab) founddelim=1;
                else {
                    if (founddelim==0) key[strlen(key)] = buffer[i];
                    else value[strlen(value)] = buffer[i];
                }
            }
            if (strlen(key)>0 && strlen(value)>0) {
                setParameter(key,value);
                if (echo) cout << key << " = " << value << endl;
            }
        }
    }
    
    return;
}

//
// Read parameters from commandline
//
void readParametersFromCommandLine(int argc, char** argv, int echo) {
    int optind = 1;
    while (optind < argc)
    {
        if (argv[optind][0]=='-') {
            string key = argv[optind];
            key = key.substr(1,key.length()); // remove '-'
            string value = argv[optind+1]; // load value
            if (echo) cout << key << " = " << value << endl;
            setParameter(key.c_str(),value.c_str());
            optind++;
        }
        optind++;
    }
    return;
}

// parameter processing prior to run
void processParameters(){}
